==> mx2g_1.1.1.tar.gz: mx-1.1.1/libmyriexpress/mx_connect.c <==
/*************************************************************************
 * The contents of this file are subject to the MYRICOM MYRINET          *
 * EXPRESS (MX) NETWORKING SOFTWARE AND DOCUMENTATION LICENSE (the       *
 * "License"); User may not use this file except in compliance with the  *
 * License.  The full text of the License can found in LICENSE.TXT       *
 *                                                                       *
 * Software distributed under the License is distributed on an "AS IS"   *
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See  *
 * the License for the specific language governing rights and            *
 * limitations under the License.                                        *
 *                                                                       *
 * Copyright 2005 by Myricom, Inc.  All rights reserved.                 *
 *************************************************************************/

#include "mx_auto_config.h"
#include "myriexpress.h"
#include "mx__lib_types.h"
#  include "mx__partner.h"
#include "mx__endpoint.h"
#include "mx__ipconnect.h"
#include "mx_byteswap.h"
#include "mx__request.h"
#include "mx__lib.h"
#include "mx__driver_interface.h"
#  include "mx_stbar.h"
#include "mx__requests.h"
#include "mx__error.h"
#include "mx__sleep.h"
#include "mx__shmem.h"

void
mx__handle_connect(struct mx_endpoint *ep, mcp_uevt_connect_t *evt)
{
  union mx_request *q;
  struct mx__partner * partner;
  partner = mx__endpoint_lookup_partner(ep, evt->source_endpt, 
					htons(evt->source_peer_index));
  mx_fixme_assert(partner);
  if (partner->connect_session_n == evt->session_id && 
      (char)(evt->connect_seqnum - partner->connect_recvseq) <= 0) {
#if MX_DEBUG
    mx_printf("Received duplicate connect packet from peer %d:"
	      "eid=%d,session=0x%08x,connect_seqnum=0x%x\n",
	      htons(evt->source_peer_index), evt->source_endpt, 
	      (int)htonl(evt->session_id), (int)htonl(evt->connect_seqnum));
#endif
    return;
  }
  q = mx__rl_alloc(&ep->req_lookaside);
  mx_fixme_assert(q);
  memset(q,0,sizeof(*q));
  if (htonl(evt->app_key) == ep->connect.key) {
    if (partner->connect_session_n != evt->session_id) {
      mx__partner_cleanup(ep, partner);
    }
    q->connect.lib_seqnum = partner->recv_seq;
    q->connect.status_code_n = MX_STATUS_SUCCESS;
    q->connect.app_key_n = ep->endpoint_sid_n;
    partner->connect_session_n = evt->session_id;
    partner->connect_recvseq = evt->connect_seqnum;
  } else {
    q->connect.status_code_n = MX_STATUS_BAD_KEY;
  }
  
  q->connect.connect_seqnum_n = evt->connect_seqnum;
  q->connect.timeout = 0;
  q->connect.dest_session_n = evt->session_id;
  q->connect.peer_endpoint_id = evt->source_endpt;
  q->connect.peer_index_n = evt->source_peer_index;
  q->connect.basic.mcp_handle = -1;
  q->connect.basic.state = MX__REQUEST_STATE_SEND_QUEUED;
  q->connect.basic.type = MX__REQUEST_TYPE_CONNECT_REPLY;
  q->connect.basic.wait_state = MX__WAIT_STATE_COMPLETE;
  mx__enqueue_request(&ep->send_reqq, q);
  /* we cannot call mx_luigi, not a problem becasuse
     mx__process_requests is currently done after mx_process_events,
     and even if not somebody will eventually call mx_luigi anyway */
}

void
mx__handle_connect_reply(struct mx_endpoint *ep, mcp_uevt_connect_t *evt)
{
  union mx_request *q;
  struct mx__partner *partner;
  int count = 0;
  for (q = ep->connectq.basic.next;q != &ep->connectq; q = q->basic.next) {
    count +=1;
    if (q->connect.connect_seqnum_n == evt->connect_seqnum && 
	evt->session_id == ep->endpoint_sid_n &&
	evt->source_endpt == q->connect.peer_endpoint_id &&
	evt->source_peer_index == q->connect.peer_index_n &&
	!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
      goto found;
    }
  }
  if (mx__opt.verbose)
    mx_printf("orphan connect reply:%x,queued=%d\n", 
	      (unsigned int)htonl(evt->connect_seqnum),count);
  return;
  
found:

  partner = mx__endpoint_lookup_partner(ep, q->connect.peer_endpoint_id, 
					htons(q->connect.peer_index_n));
  mx_fixme_assert(partner);
  q->connect.basic.status.code = evt->status_code;
  if (evt->status_code == MX_STATUS_SUCCESS && partner->endpoint_sid_n != evt->app_key) {
    /* only use the result if this is a real new session */
    partner->endpoint_sid_n = evt->app_key;
    partner->send_seq = htons(evt->lib_seqnum);
  }
  mx__wake_request(ep, q);
  q->connect.basic.state |= MX__REQUEST_STATE_COMPLETED;
}

void
mx__init_connect(struct mx_endpoint *ep, uint32_t endpoint_key, mx_param_t *param, uint32_t params_count )
{
  ep->connect.key = endpoint_key;
}

void
mx__end_connect(struct mx_endpoint *ep)
{
}


MX_FUNC(mx_return_t)
mx_connect(struct mx_endpoint *ep, uint64_t nic_id, uint32_t eid, 
	   uint32_t key, uint32_t timeout, mx_endpoint_addr_t *addrp)
{
  union mx_request *q;
  mx_return_t rc;
  mx__endpoint_addr_t *dest;
  struct mx__partner *partner;
  int code;
  mx_lookup_peer_t lookup;


  MX__MUTEX_LOCK(&ep->lock);
  q = mx__rl_alloc(&ep->req_lookaside);
  mx_fixme_assert(q);

  dest =  (mx__endpoint_addr_t*)&q->basic.status.source;
  if (mx__opt.no_myrinet && nic_id == UINT64_C(0x123456789abc)) {
    dest->peer_index_n = 0;
    lookup.index = 0;
  } else {
    lookup.board_number = ep->board_num;
    lookup.nic_id = nic_id;
    rc = mx__nic_id_to_peer_index(ep->handle, &lookup);
    if (rc != MX_SUCCESS) {
      rc = mx__error(ep, "mx_connect(%012"PRIx64":%d)", MX_NIC_NOT_FOUND, nic_id, eid);
      mx__rl_free(&ep->req_lookaside, q);
      MX__MUTEX_UNLOCK(&ep->lock);
      goto out;
    }
  }
  dest->endpoint_id = eid;
  mx_assert(lookup.index < 0xffff);
  dest->peer_index_n = htons((unsigned short)lookup.index);
  partner = mx__endpoint_lookup_partner(ep, eid, htons(dest->peer_index_n));
  mx_fixme_assert(partner);
  dest->partner = partner;

  if (partner == ep->addr.partner) {
    q->connect.basic.status.code = MX_STATUS_SUCCESS;
    goto skip_network;
  }
#if MX_USE_SHMEM
  if (!mx__opt.disable_shmem && dest->peer_index_n == ep->addr.peer_index_n) {
    struct mx__shm_peer *peer = ep->shm->peers+dest->endpoint_id;
    mx__shm_forget_peer(ep, dest->endpoint_id);
    ep->shm->peers[dest->endpoint_id].snd_shmq = mx__shm_open(ep, dest->endpoint_id, 0, 0);
    q->connect.basic.status.code = MX_STATUS_SUCCESS;
    if (!peer->snd_shmq) {
      q->connect.basic.status.code = MX_STATUS_ENDPOINT_CLOSED;
    }
    goto skip_network;
  }
#endif
  q->connect.dest_session_n = ep->endpoint_sid_n;
  q->connect.app_key_n = htonl(key);
  q->connect.connect_seqnum_n = ++partner->connect_sendseq;
  q->connect.peer_endpoint_id = dest->endpoint_id;
  q->connect.peer_index_n = dest->peer_index_n;
  /* API timeouts are in milliseconds, no comment :-( */
  q->connect.timeout = timeout == MX_INFINITE ? 0x3fffffff : timeout * 1000;
  q->connect.basic.mcp_handle = -1;
  q->connect.basic.type = MX__REQUEST_TYPE_CONNECT;
  q->connect.basic.wait_state = MX__WAIT_STATE_COMPLETE;
  q->connect.basic.state = MX__REQUEST_STATE_SEND_QUEUED;
  mx__enqueue_request(&ep->send_reqq, q);
  mx__luigi(ep);

  if (!(q->connect.basic.state & MX__REQUEST_STATE_ACKED)) {
    mx__sleep_on_request(ep, q, MX_INFINITE, &ep->waiters, NULL);
  }
  /* wait to be woken up by MCP ack or final completion */
  if (!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
    /* check we woke up for something */
    mx_assert(q->connect.basic.state == MX__REQUEST_STATE_ACKED);
    /* give it an extra timeout */
    mx__sleep_on_request(ep, q, timeout, &ep->waiters, NULL);
    if (!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
      mx_assert(q->connect.basic.state == MX__REQUEST_STATE_ACKED);
      q->connect.basic.status.code = MX_STATUS_TIMEOUT;
      q->connect.basic.state |= MX__REQUEST_STATE_COMPLETED;
    }
  } else if (!(q->connect.basic.state & MX__REQUEST_STATE_ACKED)) {
    mx_assert(q->connect.basic.state & (MX__REQUEST_STATE_MCP | MX__REQUEST_STATE_SEND_QUEUED));
    mx__sleep_on_request(ep, q, MX_INFINITE, &ep->waiters, NULL);
    mx_assert(q->connect.basic.state & MX__REQUEST_STATE_ACKED);
  } 
  mx_assert(q->connect.basic.state == (MX__REQUEST_STATE_ACKED | MX__REQUEST_STATE_COMPLETED));
  mx__spliceout_request(&ep->connectq, q);

 skip_network:
  *addrp = q->connect.basic.status.source;
  code =  q->connect.basic.status.code;
  mx__error_req(ep, "mx_connect", 0, code);
  rc = (code == MX_STATUS_SUCCESS) ? MX_SUCCESS : 
    (code == MX_STATUS_BAD_KEY) ? MX_BAD_CONNECTION_KEY  :
    (code == MX_STATUS_ENDPOINT_CLOSED) ? MX_CONNECTION_FAILED :
    (code == MX_STATUS_TIMEOUT) ? MX_TIMEOUT :
    MX_CONNECTION_FAILED;
  mx__rl_free(&ep->req_lookaside, q);
  MX__MUTEX_UNLOCK(&ep->lock);

 out:
  return rc;
}

==> mx2g_1.1.4.tar.gz: mx-1.1.4/libmyriexpress/mx_connect.c <==
/*************************************************************************
 * The contents of this file are subject to the MYRICOM MYRINET          *
 * EXPRESS (MX) NETWORKING SOFTWARE AND DOCUMENTATION LICENSE (the       *
 * "License"); User may not use this file except in compliance with the  *
 * License.  The full text of the License can found in LICENSE.TXT       *
 *                                                                       *
 * Software distributed under the License is distributed on an "AS IS"   *
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See  *
 * the License for the specific language governing rights and            *
 * limitations under the License.                                        *
 *                                                                       *
 * Copyright 2005 by Myricom, Inc.  All rights reserved.                 *
 *************************************************************************/

#include "mx_auto_config.h"
#include "myriexpress.h"
#include "mx__lib_types.h"
#  include "mx__partner.h"
#include "mx__endpoint.h"
#include "mx__ipconnect.h"
#include "mx_byteswap.h"
#include "mx__request.h"
#include "mx__lib.h"
#include "mx__driver_interface.h"
#  include "mx_stbar.h"
#include "mx__requests.h"
#include "mx__error.h"
#include "mx__sleep.h"
#include "mx__shmem.h"

void
mx__handle_connect(struct mx_endpoint *ep, mcp_uevt_connect_t *evt)
{
  union mx_request *q;
  struct mx__partner * partner;
  partner = mx__endpoint_lookup_partner(ep, evt->source_endpt, 
					htons(evt->source_peer_index));
  mx_fixme_assert(partner);
  if (partner->connect_session_n == evt->session_id && 
      (char)(evt->connect_seqnum - partner->connect_recvseq) <= 0) {
#if MX_DEBUG
    mx_printf("Received duplicate connect packet from peer %d:"
	      "eid=%d,session=0x%08x,connect_seqnum=0x%x\n",
	      htons(evt->source_peer_index), evt->source_endpt, 
	      (int)htonl(evt->session_id), (int)htonl(evt->connect_seqnum));
#endif
    return;
  }
  q = mx__rl_alloc(&ep->req_lookaside);
  mx_fixme_assert(q);
  memset(q,0,sizeof(*q));
  if (htonl(evt->app_key) == ep->connect.key) {
    if (partner->connect_session_n != evt->session_id) {
      mx__partner_cleanup(ep, partner);
    }
    q->connect.lib_seqnum = partner->recv_seq;
    q->connect.status_code_n = MX_STATUS_SUCCESS;
    q->connect.app_key_n = ep->endpoint_sid_n;
    partner->connect_session_n = evt->session_id;
    partner->connect_recvseq = evt->connect_seqnum;
  } else {
    q->connect.status_code_n = MX_STATUS_BAD_KEY;
  }
  
  q->connect.connect_seqnum_n = evt->connect_seqnum;
  q->connect.timeout = 0;
  q->connect.dest_session_n = evt->session_id;
  q->connect.peer_endpoint_id = evt->source_endpt;
  q->connect.peer_index_n = evt->source_peer_index;
  q->connect.basic.mcp_handle = -1;
  q->connect.basic.state = MX__REQUEST_STATE_SEND_QUEUED;
  q->connect.basic.type = MX__REQUEST_TYPE_CONNECT_REPLY;
  q->connect.basic.wait_state = MX__WAIT_STATE_COMPLETE;
  mx__enqueue_request(&ep->send_reqq, q);
  /* we cannot call mx_luigi, not a problem becasuse
     mx__process_requests is currently done after mx_process_events,
     and even if not somebody will eventually call mx_luigi anyway */
}

void
mx__handle_connect_reply(struct mx_endpoint *ep, mcp_uevt_connect_t *evt)
{
  union mx_request *q;
  struct mx__partner *partner;
  int count = 0;
  for (q = ep->connectq.basic.next;q != &ep->connectq; q = q->basic.next) {
    count +=1;
    if (q->connect.connect_seqnum_n == evt->connect_seqnum && 
	evt->session_id == ep->endpoint_sid_n &&
	evt->source_endpt == q->connect.peer_endpoint_id &&
	evt->source_peer_index == q->connect.peer_index_n &&
	!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
      goto found;
    }
  }
  if (mx__opt.verbose)
    mx_printf("orphan connect reply:%x,queued=%d\n", 
	      (unsigned int)htonl(evt->connect_seqnum),count);
  return;
  
found:

  partner = mx__endpoint_lookup_partner(ep, q->connect.peer_endpoint_id, 
					htons(q->connect.peer_index_n));
  mx_fixme_assert(partner);
  q->connect.basic.status.code = evt->status_code;
  if (evt->status_code == MX_STATUS_SUCCESS && partner->endpoint_sid_n != evt->app_key) {
    /* only use the result if this is a real new session */
    partner->endpoint_sid_n = evt->app_key;
    partner->send_seq = htons(evt->lib_seqnum);
  }
  mx__wake_request(ep, q);
  q->connect.basic.state |= MX__REQUEST_STATE_COMPLETED;
}

void
mx__init_connect(struct mx_endpoint *ep, uint32_t endpoint_key, mx_param_t *param, uint32_t params_count )
{
  ep->connect.key = endpoint_key;
}

void
mx__end_connect(struct mx_endpoint *ep)
{
}


MX_FUNC(mx_return_t)
mx_connect(struct mx_endpoint *ep, uint64_t nic_id, uint32_t eid, 
	   uint32_t key, uint32_t timeout, mx_endpoint_addr_t *addrp)
{
  union mx_request *q;
  mx_return_t rc;
  mx__endpoint_addr_t *dest;
  struct mx__partner *partner;
  int code;
  mx_lookup_peer_t lookup;


  MX__MUTEX_LOCK(&ep->lock);
  q = mx__rl_alloc(&ep->req_lookaside);
  mx_fixme_assert(q);
  q->connect.basic.type = MX__REQUEST_TYPE_CONNECT;

  dest =  (mx__endpoint_addr_t*)&q->basic.status.source;
  if (nic_id == ep->addr.partner->nic_id) {
    lookup.index = ntohs(ep->addr.peer_index_n);
  } else {
    lookup.board_number = ep->board_num;
    lookup.nic_id = nic_id;
    rc = mx__nic_id_to_peer_index(ep->handle, &lookup);
    if (rc != MX_SUCCESS) {
      rc = mx__error(ep, "mx_connect(%012"PRIx64":%d)", MX_NIC_NOT_FOUND, nic_id, eid);
      mx__rl_free(&ep->req_lookaside, q);
      MX__MUTEX_UNLOCK(&ep->lock);
      goto out;
    }
  }
  dest->endpoint_id = eid;
  mx_assert(lookup.index < 0xffff);
  dest->peer_index_n = htons((unsigned short)lookup.index);
  partner = mx__endpoint_lookup_partner(ep, eid, htons(dest->peer_index_n));
  mx_fixme_assert(partner);
  dest->partner = partner;

  if (partner == ep->addr.partner) {
    q->connect.basic.status.code = MX_STATUS_SUCCESS;
    goto skip_network;
  }
#if MX_USE_SHMEM
  if (!mx__opt.disable_shmem && dest->peer_index_n == ep->addr.peer_index_n) {
    struct mx__shm_peer *peer = ep->shm->peers+dest->endpoint_id;
    mx__shm_forget_peer(ep, dest->endpoint_id);
    ep->shm->peers[dest->endpoint_id].snd_shmq = mx__shm_open(ep, dest->endpoint_id, 0, 0);
    q->connect.basic.status.code = MX_STATUS_SUCCESS;
    if (!peer->snd_shmq) {
      q->connect.basic.status.code = MX_STATUS_ENDPOINT_CLOSED;
    }
    goto skip_network;
  }
#endif
  q->connect.dest_session_n = ep->endpoint_sid_n;
  q->connect.app_key_n = htonl(key);
  q->connect.connect_seqnum_n = ++partner->connect_sendseq;
  q->connect.peer_endpoint_id = dest->endpoint_id;
  q->connect.peer_index_n = dest->peer_index_n;
  /* API timeouts are in milliseconds, no comment :-( */
  q->connect.timeout = timeout == MX_INFINITE ? 0x3fffffff : timeout * 1000;
  q->connect.basic.mcp_handle = -1;
  q->connect.basic.type = MX__REQUEST_TYPE_CONNECT;
  q->connect.basic.wait_state = MX__WAIT_STATE_COMPLETE;
  q->connect.basic.state = MX__REQUEST_STATE_SEND_QUEUED;
  mx__enqueue_request(&ep->send_reqq, q);
  mx__luigi(ep);

  if (!(q->connect.basic.state & MX__REQUEST_STATE_ACKED)) {
    mx__sleep_on_request(ep, q, MX_INFINITE, &ep->waiters, NULL);
  }
  /* wait to be woken up by MCP ack or final completion */
  if (!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
    /* check we woke up for something */
    mx_assert(q->connect.basic.state == MX__REQUEST_STATE_ACKED);
    /* give it an extra timeout */
    mx__sleep_on_request(ep, q, timeout, &ep->waiters, NULL);
    if (!(q->connect.basic.state & MX__REQUEST_STATE_COMPLETED)) {
      mx_assert(q->connect.basic.state == MX__REQUEST_STATE_ACKED);
      q->connect.basic.status.code = MX_STATUS_TIMEOUT;
      q->connect.basic.state |= MX__REQUEST_STATE_COMPLETED;
    }
  } else if (!(q->connect.basic.state & MX__REQUEST_STATE_ACKED)) {
    mx_assert(q->connect.basic.state & (MX__REQUEST_STATE_MCP | MX__REQUEST_STATE_SEND_QUEUED));
    mx__sleep_on_request(ep, q, MX_INFINITE, &ep->waiters, NULL);
    mx_assert(q->connect.basic.state & MX__REQUEST_STATE_ACKED);
  } 
  mx_assert(q->connect.basic.state == (MX__REQUEST_STATE_ACKED | MX__REQUEST_STATE_COMPLETED));
  mx__spliceout_request(&ep->connectq, q);

 skip_network:
  *addrp = q->connect.basic.status.source;
  code =  q->connect.basic.status.code;
  mx__error_req(ep, "mx_connect", q, code);
  rc = (code == MX_STATUS_SUCCESS) ? MX_SUCCESS : 
    (code == MX_STATUS_BAD_KEY) ? MX_BAD_CONNECTION_KEY  :
    (code == MX_STATUS_ENDPOINT_CLOSED) ? MX_CONNECTION_FAILED :
    (code == MX_STATUS_TIMEOUT) ? MX_TIMEOUT :
    MX_CONNECTION_FAILED;
  mx__rl_free(&ep->req_lookaside, q);
  MX__MUTEX_UNLOCK(&ep->lock);

 out:
  return rc;
}
